package weka.classifiers.TzuCheng.tpst;

import weka.core.matrix.*;

public class MatrixExt extends Matrix{
  /** for serialization */
  static final long serialVersionUID = -8312249377322734193L;
  
  
  //protected Matrix m_matrix;
  
 
  
  public MatrixExt(Matrix matrix_input) {
    super(matrix_input.getArray());
  }
  
  
  public MatrixExt(double [][] double_array) {
    //m_matrix=new Matrix(double_array);
    super(double_array);
  }
  
  /*
   * Duplicate Matlab cov function
   * find the covariance matrix
  */
  public Matrix cov(Matrix source){
    
    double source_num_row=source.getRowDimension();
    Matrix x_mean=mean(source);
    Matrix x_minus_mean=source.minus(repmat(x_mean,source.getRowDimension(),1));
    Matrix x_minus_mean_transpose=new Matrix(x_minus_mean.transpose().getArray());
    
    //System.out.println(x_minus_mean);
    
    Matrix ans=x_minus_mean_transpose.times(x_minus_mean);
    double ratio=1/(source_num_row-1);
    ans=ans.times(ratio);
    return new Matrix(ans.getArray());
  }
  
  
  /*
   * Duplicate Matlab repmat function
   * rep_row : number of repetition on rows
   * rep_col : number of repetition on columns
   * For example repmat (A,2,3) would be
   * 
   *  | A A A |
   *  | A A A |
   *   
   * */
  public Matrix repmat(Matrix source,int rep_row,int rep_col)  {
    int init_row=source.getRowDimension();
    int init_col=source.getColumnDimension();
    double [][]double_array;
    double_array= new double[rep_row*init_row][];
    for (int i=0;i<rep_row*init_row;i++){
      double_array[i]=new double[rep_col*init_col];
    }
    for (int i=0;i<rep_row*init_row;i++){
      for (int j=0;j<rep_col*init_col;j++){
	double_array[i][j]=source.get(i%init_row,j%init_col);
      }
    }

    return new Matrix(double_array);
    
  }
  /*
   * Try to duplicate Matlab mean function
   * direction=0 -> means along column
   * direction=1 -> means along row
   * Usually, each instance is a row vector, so it makes sense
   * to find the mean along the column by default.
   * */
  public Matrix mean(Matrix source)  {
    return mean (source,0);  
  }
  public Matrix mean (Matrix source,int direction)  {
    Matrix matrix_inClass= source.copy();
    double [][]double_array=matrix_inClass.getArray();
    double []sum_along_column= new double[matrix_inClass.getColumnDimension()];
    for(int i=0;i<matrix_inClass.getColumnDimension();i++){
      for(int j=0;j<matrix_inClass.getRowDimension();j++)
	sum_along_column[i]=sum_along_column[i]+double_array[j][i];
    }
    
    double [][]avg_along_column =new double[1][matrix_inClass.getColumnDimension()];
 
    for(int i=0;i<matrix_inClass.getColumnDimension();i++){
      avg_along_column[0][i]= sum_along_column[i]/matrix_inClass.getRowDimension(); 
    }
    
    return new Matrix(avg_along_column);
    
  }
  
  
  
  public Matrix pinv(Matrix mB){
     
    try{
      mB=mB.inverse();
      return mB;
    }catch(Exception e){//Singular Value Decomposition to get its pseudo inverse, there is error
      System.out.println("Matrix is singular");
      SingularValueDecomposition svd_result= mB.svd();
    
      Matrix mU=new Matrix (svd_result.getU().getArray());
      Matrix mS=new Matrix (svd_result.getS().getArray(),mB.getRowDimension(),mB.getColumnDimension());
      Matrix mV=new Matrix (svd_result.getV().getArray());  
    
      Matrix ans=mV.times(reciprocal(mS,svd_result.rank()));
      ans=ans.times(mU.transpose());
      return ans;
    }
  	//System.out.println(MC);
  
  }


  public Matrix reciprocal(Matrix source,int rank){
    Matrix ans=new Matrix(source.getColumnDimension(),source.getRowDimension());
    for (int i=0;i<rank;i++){
      ans.set(i, i, 1/source.get(i,i));
    }
    return ans;
  } 

}
